package kr.co.vividnext.sodalive.admin.event

import kr.co.vividnext.sodalive.common.SodaException
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import java.time.LocalDate
import java.time.ZoneId
import java.time.format.DateTimeFormatter

@Service
@Transactional(readOnly = true)
class AdminChargeEventService(private val repository: AdminChargeEventRepository) {
    @Transactional
    fun createChargeEvent(request: CreateChargeEventRequest): Long {
        val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd")
        val startDate = LocalDate.parse(request.startDateString, dateTimeFormatter).atTime(0, 0)
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        val endDate = LocalDate.parse(request.endDateString, dateTimeFormatter).atTime(23, 59, 59)
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        val chargeEvent = ChargeEvent(
            title = request.title,
            startDate = startDate,
            endDate = endDate,
            availableCount = request.availableCount,
            addPercent = request.addPercent / 100f
        )

        return repository.save(chargeEvent).id!!
    }

    @Transactional
    fun modifyChargeEvent(request: ModifyChargeEventRequest) {
        val chargeEvent = repository.findByIdOrNull(request.id)
            ?: throw SodaException("해당하는 충전이벤트가 없습니다\n다시 시도해 주세요.")

        if (request.title != null) {
            chargeEvent.title = request.title
        }

        val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd")
        if (request.startDateString != null) {
            chargeEvent.startDate = LocalDate.parse(request.startDateString, dateTimeFormatter).atTime(0, 0)
                .atZone(ZoneId.of("Asia/Seoul"))
                .withZoneSameInstant(ZoneId.of("UTC"))
                .toLocalDateTime()
        }

        if (request.endDateString != null) {
            chargeEvent.endDate = LocalDate.parse(request.endDateString, dateTimeFormatter).atTime(23, 59, 59)
                .atZone(ZoneId.of("Asia/Seoul"))
                .withZoneSameInstant(ZoneId.of("UTC"))
                .toLocalDateTime()
        }

        if (request.availableCount != null) {
            chargeEvent.availableCount = request.availableCount
        }

        if (request.addPercent != null) {
            chargeEvent.addPercent = request.addPercent / 100f
        }

        if (request.isActive != null) {
            chargeEvent.isActive = request.isActive
        }
    }

    fun getChargeEventList(): List<GetChargeEventListResponse> {
        return repository.getChargeEventList()
            .map {
                val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd")
                val startDate = it.startDate
                    .atZone(ZoneId.of("UTC"))
                    .withZoneSameInstant(ZoneId.of("Asia/Seoul"))
                    .format(dateTimeFormatter)

                val endDate = it.endDate
                    .atZone(ZoneId.of("UTC"))
                    .withZoneSameInstant(ZoneId.of("Asia/Seoul"))
                    .format(dateTimeFormatter)

                GetChargeEventListResponse(
                    id = it.id!!,
                    title = it.title,
                    startDate = startDate,
                    endDate = endDate,
                    availableCount = it.availableCount,
                    addPercent = (it.addPercent * 100).toInt(),
                    isActive = it.isActive
                )
            }
    }
}
