package kr.co.vividnext.sodalive.content.series

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.hashtag.QHashTag.hashTag
import kr.co.vividnext.sodalive.content.series.content.GetSeriesContentMinMaxPriceResponse
import kr.co.vividnext.sodalive.content.series.content.QGetSeriesContentMinMaxPriceResponse
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeries.series
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeriesContent.seriesContent
import kr.co.vividnext.sodalive.creator.admin.content.series.Series
import kr.co.vividnext.sodalive.creator.admin.content.series.keyword.QSeriesKeyword.seriesKeyword
import org.springframework.data.jpa.repository.JpaRepository

interface ContentSeriesRepository : JpaRepository<Series, Long>, ContentSeriesQueryRepository

interface ContentSeriesQueryRepository {
    fun getSeriesTotalCount(creatorId: Long, isAuth: Boolean): Int
    fun getSeriesList(
        imageHost: String,
        creatorId: Long,
        isAuth: Boolean,
        offset: Long,
        limit: Long
    ): List<Series>

    fun getSeriesDetail(seriesId: Long, isAuth: Boolean): Series?
    fun getKeywordList(seriesId: Long): List<String>
    fun getSeriesContentMinMaxPrice(seriesId: Long): GetSeriesContentMinMaxPriceResponse
    fun getRecommendSeriesList(isAuth: Boolean, contentType: ContentType, limit: Long): List<Series>
}

class ContentSeriesQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : ContentSeriesQueryRepository {
    override fun getSeriesTotalCount(creatorId: Long, isAuth: Boolean): Int {
        var where = series.member.id.eq(creatorId)
            .and(series.isActive.isTrue)

        if (!isAuth) {
            where = where.and(series.isAdult.isFalse)
        }

        return queryFactory
            .select(series.id)
            .from(series)
            .where(where)
            .fetch()
            .size
    }

    override fun getSeriesList(
        imageHost: String,
        creatorId: Long,
        isAuth: Boolean,
        offset: Long,
        limit: Long
    ): List<Series> {
        var where = series.member.id.eq(creatorId)
            .and(series.isActive.isTrue)

        if (!isAuth) {
            where = where.and(series.isAdult.isFalse)
        }

        return queryFactory
            .selectFrom(series)
            .where(where)
            .orderBy(series.orders.asc(), series.createdAt.asc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    override fun getSeriesDetail(seriesId: Long, isAuth: Boolean): Series? {
        var where = series.id.eq(seriesId)
            .and(series.isActive.isTrue)

        if (!isAuth) {
            where = where.and(series.isAdult.isFalse)
        }

        return queryFactory
            .selectFrom(series)
            .where(where)
            .fetchFirst()
    }

    override fun getKeywordList(seriesId: Long): List<String> {
        return queryFactory
            .select(hashTag.tag)
            .from(series)
            .innerJoin(series.keywordList, seriesKeyword)
            .innerJoin(seriesKeyword.keyword, hashTag)
            .where(series.id.eq(seriesId))
            .fetch()
    }

    override fun getSeriesContentMinMaxPrice(seriesId: Long): GetSeriesContentMinMaxPriceResponse {
        return queryFactory
            .select(
                QGetSeriesContentMinMaxPriceResponse(
                    audioContent.price.min(),
                    audioContent.price.max()
                )
            )
            .from(series)
            .innerJoin(series.contentList, seriesContent)
            .innerJoin(seriesContent.content, audioContent)
            .where(series.id.eq(seriesId))
            .fetchFirst()
    }

    override fun getRecommendSeriesList(isAuth: Boolean, contentType: ContentType, limit: Long): List<Series> {
        var where = series.isActive.isTrue

        if (!isAuth) {
            where = where.and(series.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    series.member.auth.gender.eq(if (contentType == ContentType.MALE) 0 else 1)
                )
            }
        }

        return queryFactory
            .selectFrom(series)
            .where(where)
            .orderBy(Expressions.numberTemplate(Double::class.java, "function('rand')").asc())
            .offset(0)
            .limit(limit)
            .fetch()
    }
}
