package kr.co.vividnext.sodalive.content.series.content

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeries.series
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeriesContent.seriesContent
import kr.co.vividnext.sodalive.creator.admin.content.series.SeriesContent
import kr.co.vividnext.sodalive.creator.admin.content.series.SeriesSortType
import org.springframework.data.jpa.repository.JpaRepository
import java.time.LocalDateTime

interface ContentSeriesContentRepository : JpaRepository<SeriesContent, Long>, ContentSeriesContentQueryRepository

interface ContentSeriesContentQueryRepository {
    fun getContentCount(seriesId: Long, isAdult: Boolean): Int
    fun getContentList(
        seriesId: Long,
        isAdult: Boolean,
        imageHost: String,
        sortType: SeriesSortType,
        offset: Long,
        limit: Long
    ): List<GetSeriesContentListItem>

    fun isNewContent(seriesId: Long, isAdult: Boolean, fromDate: LocalDateTime, nowDate: LocalDateTime): Boolean
}

class ContentSeriesContentQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : ContentSeriesContentQueryRepository {
    override fun getContentCount(seriesId: Long, isAdult: Boolean): Int {
        var where = seriesContent.series.id.eq(seriesId)
            .and(seriesContent.content.isActive.isTrue)

        if (!isAdult) {
            where = where.and(seriesContent.content.isAdult.isFalse)
        }

        return queryFactory
            .select(seriesContent.id)
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(seriesContent.content, audioContent)
            .where(where)
            .fetch()
            .size
    }

    override fun getContentList(
        seriesId: Long,
        isAdult: Boolean,
        imageHost: String,
        sortType: SeriesSortType,
        offset: Long,
        limit: Long
    ): List<GetSeriesContentListItem> {
        var where = series.id.eq(seriesId)
            .and(audioContent.isActive.isTrue)
            .and(audioContent.duration.isNotNull)

        var sortOrders = listOf(audioContent.releaseDate.desc(), audioContent.id.desc())

        if (sortType == SeriesSortType.OLDEST) {
            sortOrders = listOf(audioContent.releaseDate.asc(), audioContent.id.asc())
        }

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        val formattedDate = Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                audioContent.releaseDate,
                "UTC",
                "Asia/Seoul"
            ),
            "%y.%m.%d"
        )

        return queryFactory
            .select(
                QGetSeriesContentListItem(
                    audioContent.id,
                    audioContent.title,
                    audioContent.coverImage.prepend("/").prepend(imageHost),
                    formattedDate,
                    audioContent.duration,
                    audioContent.price,
                    Expressions.asBoolean(false),
                    Expressions.asBoolean(false)
                )
            )
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(seriesContent.content, audioContent)
            .where(where)
            .orderBy(*sortOrders.toTypedArray())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    override fun isNewContent(
        seriesId: Long,
        isAdult: Boolean,
        fromDate: LocalDateTime,
        nowDate: LocalDateTime
    ): Boolean {
        var where = seriesContent.series.id.eq(seriesId)
            .and(seriesContent.content.isActive.isTrue)
            .and(seriesContent.content.releaseDate.after(fromDate))
            .and(seriesContent.content.releaseDate.before(nowDate))

        if (!isAdult) {
            where = where.and(seriesContent.content.isAdult.isFalse)
        }

        val itemCount = queryFactory
            .select(seriesContent.id)
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(seriesContent.content, audioContent)
            .where(where)
            .fetch()
            .size

        return itemCount > 0
    }
}
