package kr.co.vividnext.sodalive.creator.admin.content

import com.querydsl.core.types.dsl.DateTimePath
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.core.types.dsl.StringTemplate
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.AudioContent
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.hashtag.QAudioContentHashTag.audioContentHashTag
import kr.co.vividnext.sodalive.content.hashtag.QHashTag.hashTag
import kr.co.vividnext.sodalive.content.theme.QAudioContentTheme.audioContentTheme
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository
import java.time.LocalDateTime

interface CreatorAdminContentRepository : JpaRepository<AudioContent, Long>, CreatorAdminAudioContentQueryRepository

interface CreatorAdminAudioContentQueryRepository {
    fun getAudioContentTotalCount(memberId: Long, searchWord: String = ""): Int
    fun getAudioContentList(
        memberId: Long,
        offset: Long,
        limit: Long,
        searchWord: String = ""
    ): List<GetCreatorAdminContentListItem>

    fun getHashTagList(audioContentId: Long): List<String>

    fun getAudioContent(memberId: Long, audioContentId: Long): AudioContent?
}

class CreatorAdminAudioContentQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : CreatorAdminAudioContentQueryRepository {
    override fun getAudioContentTotalCount(memberId: Long, searchWord: String): Int {
        var where = audioContent.duration.isNotNull
            .and(audioContent.member.isNotNull)
            .and(audioContent.isActive.isTrue.or(audioContent.releaseDate.isNotNull))
            .and(audioContent.member.id.eq(memberId))

        if (searchWord.trim().length > 1) {
            where = where.and(
                audioContent.title.contains(searchWord)
                    .or(audioContent.member.nickname.contains(searchWord))
            )
        }

        return queryFactory
            .select(audioContent.id)
            .from(audioContent)
            .where(where)
            .fetch()
            .size
    }

    override fun getAudioContentList(
        memberId: Long,
        offset: Long,
        limit: Long,
        searchWord: String
    ): List<GetCreatorAdminContentListItem> {
        var where = audioContent.duration.isNotNull
            .and(audioContent.member.isNotNull)
            .and(audioContent.isActive.isTrue.or(audioContent.releaseDate.isNotNull))
            .and(audioContent.member.id.eq(memberId))

        if (searchWord.trim().length > 1) {
            where = where.and(
                audioContent.title.contains(searchWord)
                    .or(audioContent.member.nickname.contains(searchWord))
            )
        }

        return queryFactory
            .select(
                QGetCreatorAdminContentListItem(
                    audioContent.id,
                    audioContent.title,
                    audioContent.detail,
                    audioContent.coverImage,
                    audioContent.member!!.nickname,
                    audioContentTheme.theme,
                    audioContent.price,
                    audioContent.limited,
                    audioContent.remaining,
                    audioContent.isAdult,
                    audioContent.isCommentAvailable,
                    audioContent.duration,
                    audioContent.content,
                    formattedDateExpression(audioContent.createdAt),
                    formattedDateExpression(audioContent.releaseDate, "%Y-%m-%d %H:%i")
                )
            )
            .from(audioContent)
            .innerJoin(audioContent.theme, audioContentTheme)
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(audioContent.releaseDate.desc())
            .fetch()
    }

    override fun getHashTagList(audioContentId: Long): List<String> {
        return queryFactory
            .select(hashTag.tag)
            .from(audioContentHashTag)
            .innerJoin(audioContentHashTag.hashTag, hashTag)
            .innerJoin(audioContentHashTag.audioContent, audioContent)
            .where(
                audioContent.duration.isNotNull
                    .and(audioContent.member.isNotNull)
                    .and(audioContentHashTag.audioContent.id.eq(audioContentId))
            )
            .fetch()
    }

    override fun getAudioContent(memberId: Long, audioContentId: Long): AudioContent? {
        return queryFactory
            .selectFrom(audioContent)
            .innerJoin(audioContent.member, member)
            .where(
                member.id.eq(memberId)
                    .and(audioContent.id.eq(audioContentId))
            )
            .orderBy(audioContent.id.desc())
            .fetchFirst()
    }

    private fun formattedDateExpression(
        dateTime: DateTimePath<LocalDateTime>,
        format: String = "%Y-%m-%d"
    ): StringTemplate {
        return Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                dateTime,
                "UTC",
                "Asia/Seoul"
            ),
            format
        )
    }
}
