package kr.co.vividnext.sodalive.creator.admin.content.series

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeries.series
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeriesContent.seriesContent
import kr.co.vividnext.sodalive.creator.admin.content.series.content.QSearchContentNotInSeriesResponse
import kr.co.vividnext.sodalive.creator.admin.content.series.content.SearchContentNotInSeriesResponse
import org.springframework.data.jpa.repository.JpaRepository

interface CreatorAdminContentSeriesRepository : JpaRepository<Series, Long>, CreatorAdminContentSeriesQueryRepository

interface CreatorAdminContentSeriesQueryRepository {
    fun findByIdAndCreatorId(id: Long, creatorId: Long): Series?
    fun getSeriesCount(creatorId: Long): Int
    fun getSeriesList(
        offset: Long,
        limit: Long,
        creatorId: Long
    ): List<Series>

    fun getSeriesContentCount(creatorId: Long): Int
    fun getSeriesContentList(
        offset: Long,
        limit: Long,
        seriesId: Long,
        creatorId: Long,
        imageHost: String
    ): List<GetCreatorAdminContentSeriesContentItem>

    fun searchContentNotInSeries(
        seriesId: Long,
        searchWord: String,
        memberId: Long,
        imageHost: String
    ): List<SearchContentNotInSeriesResponse>
}

class CreatorAdminContentSeriesQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : CreatorAdminContentSeriesQueryRepository {
    override fun findByIdAndCreatorId(id: Long, creatorId: Long): Series? {
        return queryFactory
            .selectFrom(series)
            .where(
                series.id.eq(id)
                    .and(series.member.id.eq(creatorId))
            )
            .fetchFirst()
    }

    override fun getSeriesCount(creatorId: Long): Int {
        return queryFactory
            .select(series.id)
            .from(series)
            .where(
                series.member.id.eq(creatorId)
                    .and(series.isActive.isTrue)
            )
            .fetch()
            .size
    }

    override fun getSeriesList(
        offset: Long,
        limit: Long,
        creatorId: Long
    ): List<Series> {
        return queryFactory
            .selectFrom(series)
            .where(
                series.member.id.eq(creatorId)
                    .and(series.isActive.isTrue)
            )
            .orderBy(series.orders.asc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    override fun getSeriesContentCount(creatorId: Long): Int {
        return queryFactory
            .select(seriesContent.id)
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(seriesContent.content, audioContent)
            .where(
                series.member.id.eq(creatorId)
                    .and(audioContent.member.id.eq(creatorId))
                    .and(series.isActive.isTrue)
            )
            .fetch()
            .size
    }

    override fun getSeriesContentList(
        offset: Long,
        limit: Long,
        seriesId: Long,
        creatorId: Long,
        imageHost: String
    ): List<GetCreatorAdminContentSeriesContentItem> {
        return queryFactory
            .select(
                QGetCreatorAdminContentSeriesContentItem(
                    audioContent.id,
                    audioContent.coverImage.prepend("/").prepend(imageHost),
                    audioContent.title,
                    audioContent.isAdult
                )
            )
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(seriesContent.content, audioContent)
            .where(
                series.member.id.eq(creatorId)
                    .and(series.id.eq(seriesId))
                    .and(audioContent.member.id.eq(creatorId))
                    .and(series.isActive.isTrue)
            )
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    override fun searchContentNotInSeries(
        seriesId: Long,
        searchWord: String,
        memberId: Long,
        imageHost: String
    ): List<SearchContentNotInSeriesResponse> {
        return queryFactory
            .select(
                QSearchContentNotInSeriesResponse(
                    audioContent.id,
                    audioContent.title,
                    audioContent.coverImage.prepend("/").prepend(imageHost)
                )
            )
            .from(audioContent)
            .leftJoin(seriesContent)
            .on(
                audioContent.id.eq(seriesContent.content.id)
                    .and(seriesContent.series.id.eq(seriesId))
            )
            .where(
                audioContent.duration.isNotNull
                    .and(audioContent.member.isNotNull)
                    .and(audioContent.member.id.eq(memberId))
                    .and(audioContent.isActive.isTrue.or(audioContent.releaseDate.isNotNull))
                    .and(audioContent.title.contains(searchWord))
                    .and(seriesContent.id.isNull)
            )
            .fetch()
    }
}
