package kr.co.vividnext.sodalive.creator.admin.content.series

import kr.co.vividnext.sodalive.admin.content.series.genre.SeriesGenre
import kr.co.vividnext.sodalive.common.BaseEntity
import kr.co.vividnext.sodalive.creator.admin.content.series.keyword.SeriesKeyword
import kr.co.vividnext.sodalive.member.Member
import javax.persistence.CascadeType
import javax.persistence.CollectionTable
import javax.persistence.Column
import javax.persistence.ElementCollection
import javax.persistence.Entity
import javax.persistence.EnumType
import javax.persistence.Enumerated
import javax.persistence.FetchType
import javax.persistence.JoinColumn
import javax.persistence.ManyToOne
import javax.persistence.OneToMany
import javax.persistence.OneToOne

enum class SeriesPublishedDaysOfWeek {
    SUN, MON, TUE, WED, THU, FRI, SAT, RANDOM
}

enum class SeriesState {
    PROCEEDING, SUSPEND, COMPLETE
}

enum class SeriesSortType {
    NEWEST, OLDEST
}

@Entity
data class Series(
    var title: String,
    @Column(columnDefinition = "TEXT", nullable = false)
    var introduction: String,
    @Enumerated(value = EnumType.STRING)
    var state: SeriesState = SeriesState.PROCEEDING,
    var writer: String? = null,
    var studio: String? = null,
    @ElementCollection(targetClass = SeriesPublishedDaysOfWeek::class, fetch = FetchType.EAGER)
    @Enumerated(value = EnumType.STRING)
    @CollectionTable(name = "series_published_days_of_week", joinColumns = [JoinColumn(name = "series_id")])
    val publishedDaysOfWeek: MutableSet<SeriesPublishedDaysOfWeek> = mutableSetOf(),
    var isAdult: Boolean = false,
    var isActive: Boolean = true,
    var orders: Int = 1
) : BaseEntity() {
    @OneToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "genre_id", nullable = false)
    var genre: SeriesGenre? = null

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "member_id", nullable = false)
    var member: Member? = null

    var coverImage: String? = null

    @OneToMany(mappedBy = "series", cascade = [CascadeType.ALL], orphanRemoval = true)
    var contentList: MutableList<SeriesContent> = mutableListOf()

    @OneToMany(mappedBy = "series", cascade = [CascadeType.ALL], orphanRemoval = true)
    var keywordList: MutableList<SeriesKeyword> = mutableListOf()

    fun toDetailResponse(imageHost: String): GetCreatorAdminContentSeriesDetailResponse {
        return GetCreatorAdminContentSeriesDetailResponse(
            seriesId = id!!,
            title = title,
            introduction = introduction,
            coverImageUrl = "$imageHost/${coverImage!!}",
            publishedDaysOfWeek = publishedDaysOfWeekText(),
            genre = genre!!.genre,
            keywords = keywordList.map { it.keyword!!.tag }.joinToString(" ") { it },
            isAdult = isAdult,
            state = stateSeriesText(),
            writer = writer,
            studio = studio
        )
    }

    private fun publishedDaysOfWeekText(): String {
        return publishedDaysOfWeek.toList().sortedBy { it.ordinal }.map {
            when (it) {
                SeriesPublishedDaysOfWeek.SUN -> "일"
                SeriesPublishedDaysOfWeek.MON -> "월"
                SeriesPublishedDaysOfWeek.TUE -> "화"
                SeriesPublishedDaysOfWeek.WED -> "수"
                SeriesPublishedDaysOfWeek.THU -> "목"
                SeriesPublishedDaysOfWeek.FRI -> "금"
                SeriesPublishedDaysOfWeek.SAT -> "토"
                SeriesPublishedDaysOfWeek.RANDOM -> "랜덤"
            }
        }.joinToString(", ") { it }
    }

    private fun stateSeriesText(): String {
        return when (state) {
            SeriesState.PROCEEDING -> "연재중"
            SeriesState.SUSPEND -> "휴재중"
            SeriesState.COMPLETE -> "완결"
        }
    }
}
