package kr.co.vividnext.sodalive.member.block

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.member.QMember.member
import kr.co.vividnext.sodalive.member.block.QBlockMember.blockMember
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface BlockMemberRepository : JpaRepository<BlockMember, Long>, BlockMemberQueryRepository

interface BlockMemberQueryRepository {
    fun getBlockAccount(blockedMemberId: Long, memberId: Long): BlockMember?
    fun isBlocked(blockedMemberId: Long, memberId: Long): Boolean
    fun getBlockedMemberIdList(memberId: Long): List<Long>
    fun getBlockedMemberList(offset: Long, limit: Long, memberId: Long): List<GetBlockedMemberListItem>
    fun getBlockedMemberTotalCount(memberId: Long): Int
}

@Repository
class BlockMemberQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) : BlockMemberQueryRepository {
    override fun getBlockAccount(blockedMemberId: Long, memberId: Long): BlockMember? {
        return queryFactory
            .selectFrom(blockMember)
            .where(
                blockMember.blockedMember.id.eq(blockedMemberId)
                    .and(blockMember.member.id.eq(memberId))
            )
            .orderBy(blockMember.id.desc())
            .fetchFirst()
    }

    override fun isBlocked(blockedMemberId: Long, memberId: Long): Boolean {
        val blockedAccount = queryFactory
            .select(blockMember.id)
            .from(blockMember)
            .where(
                blockMember.member.id.eq(memberId)
                    .and(blockMember.blockedMember.id.eq(blockedMemberId))
                    .and(blockMember.isActive.isTrue)
            )
            .fetchOne()

        return blockedAccount != null
    }

    override fun getBlockedMemberIdList(memberId: Long): List<Long> {
        return queryFactory
            .select(blockMember.blockedMember.id)
            .from(blockMember)
            .where(
                blockMember.member.id.eq(memberId)
                    .and(blockMember.isActive.isTrue)
            )
            .fetch()
    }

    override fun getBlockedMemberList(offset: Long, limit: Long, memberId: Long): List<GetBlockedMemberListItem> {
        return queryFactory
            .select(
                QGetBlockedMemberListItem(
                    blockMember.blockedMember.id,
                    blockMember.blockedMember.nickname,
                    blockMember.blockedMember.profileImage.prepend("/").prepend(cloudFrontHost),
                    blockMember.isActive
                )
            )
            .from(blockMember)
            .innerJoin(blockMember.blockedMember, member)
            .where(
                blockMember.member.id.eq(memberId)
                    .and(blockMember.isActive.isTrue)
                    .and(blockMember.blockedMember.isActive.isTrue)
            )
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    override fun getBlockedMemberTotalCount(memberId: Long): Int {
        return queryFactory
            .select(blockMember.id)
            .from(blockMember)
            .where(
                blockMember.member.id.eq(memberId)
                    .and(blockMember.isActive.isTrue)
                    .and(blockMember.blockedMember.isActive.isTrue)
            )
            .fetch()
            .size
    }
}
